@@ -42,6 +42,7 @@ typedef struct unpack_user {
 
 #define msgpack_unpack_user unpack_user
 
+typedef int (*execute_fn)(msgpack_unpack_struct(_context)* ctx, const char* data, size_t len, size_t* off);
 
 struct template_context;
 typedef struct template_context template_context;
@@ -162,6 +163,8 @@ static inline int template_callback_array_end(unpack_user* u, msgpack_unpack_obj
 {
     if (u->list_hook) {
         PyObject *new_c = PyEval_CallFunction(u->list_hook, "(O)", *c);
+        if (!new_c)
+            return -1;
         Py_DECREF(*c);
         *c = new_c;
     }
@@ -206,9 +209,9 @@ static inline int template_callback_map_end(unpack_user* u, msgpack_unpack_objec
 {
     if (u->object_hook) {
         PyObject *new_c = PyEval_CallFunction(u->object_hook, "(O)", *c);
-        if (new_c == NULL) {
+        if (!new_c)
             return -1;
-        }
+
         Py_DECREF(*c);
         *c = new_c;
     }
