@@ -3,9 +3,7 @@
 
 import six
 import struct
-from nose import main
-from nose.tools import *
-from nose.plugins.skip import SkipTest
+from pytest import raises, xfail
 
 from msgpack import packb, unpackb, Unpacker, Packer
 
@@ -13,7 +11,7 @@ from io import BytesIO
 
 def check(data, use_list=False):
     re = unpackb(packb(data), use_list=use_list)
-    assert_equal(re, data)
+    assert re == data
 
 def testPack():
     test_data = [
@@ -35,11 +33,11 @@ def testPackUnicode():
         ]
     for td in test_data:
         re = unpackb(packb(td, encoding='utf-8'), use_list=1, encoding='utf-8')
-        assert_equal(re, td)
+        assert re == td
         packer = Packer(encoding='utf-8')
         data = packer.pack(td)
         re = Unpacker(BytesIO(data), encoding='utf-8', use_list=1).unpack()
-        assert_equal(re, td)
+        assert re == td
 
 def testPackUTF32():
     try:
@@ -51,9 +49,9 @@ def testPackUTF32():
             ]
         for td in test_data:
             re = unpackb(packb(td, encoding='utf-32'), use_list=1, encoding='utf-32')
-            assert_equal(re, td)
-    except LookupError:
-        raise SkipTest
+            assert re == td
+    except LookupError as e:
+        xfail(e)
 
 def testPackBytes():
     test_data = [
@@ -64,31 +62,73 @@ def testPackBytes():
 
 def testIgnoreUnicodeErrors():
     re = unpackb(packb(b'abc\xeddef'), encoding='utf-8', unicode_errors='ignore', use_list=1)
-    assert_equal(re, "abcdef")
+    assert re == "abcdef"
 
-@raises(UnicodeDecodeError)
 def testStrictUnicodeUnpack():
-    unpackb(packb(b'abc\xeddef'), encoding='utf-8', use_list=1)
+    with raises(UnicodeDecodeError):
+        unpackb(packb(b'abc\xeddef'), encoding='utf-8', use_list=1)
 
-@raises(UnicodeEncodeError)
 def testStrictUnicodePack():
-    packb(six.u("abc\xeddef"), encoding='ascii', unicode_errors='strict')
+    with raises(UnicodeEncodeError):
+        packb(six.u("abc\xeddef"), encoding='ascii', unicode_errors='strict')
 
 def testIgnoreErrorsPack():
     re = unpackb(packb(six.u("abcФФФdef"), encoding='ascii', unicode_errors='ignore'), encoding='utf-8', use_list=1)
-    assert_equal(re, six.u("abcdef"))
+    assert re == six.u("abcdef")
 
-@raises(TypeError)
 def testNoEncoding():
-    packb(six.u("abc"), encoding=None)
+    with raises(TypeError):
+        packb(six.u("abc"), encoding=None)
 
 def testDecodeBinary():
     re = unpackb(packb("abc"), encoding=None, use_list=1)
-    assert_equal(re, b"abc")
+    assert re == b"abc"
 
 def testPackFloat():
-    assert_equal(packb(1.0, use_single_float=True),  b'\xca' + struct.pack('>f', 1.0))
-    assert_equal(packb(1.0, use_single_float=False), b'\xcb' + struct.pack('>d', 1.0))
+    assert packb(1.0, use_single_float=True)  == b'\xca' + struct.pack('>f', 1.0)
+    assert packb(1.0, use_single_float=False) == b'\xcb' + struct.pack('>d', 1.0)
+
+def testArraySize(sizes=[0, 5, 50, 1000]):
+    bio = six.BytesIO()
+    packer = Packer()
+    for size in sizes:
+        bio.write(packer.pack_array_header(size))
+        for i in range(size):
+            bio.write(packer.pack(i))
+
+    bio.seek(0)
+    unpacker = Unpacker(bio, use_list=1)
+    for size in sizes:
+        assert unpacker.unpack() == list(range(size))
+
+def test_manualreset(sizes=[0, 5, 50, 1000]):
+    packer = Packer(autoreset=False)
+    for size in sizes:
+        packer.pack_array_header(size)
+        for i in range(size):
+            packer.pack(i)
+
+    bio = six.BytesIO(packer.bytes())
+    unpacker = Unpacker(bio, use_list=1)
+    for size in sizes:
+        assert unpacker.unpack() == list(range(size))
+
+    packer.reset()
+    assert packer.bytes() == b''
+
+def testMapSize(sizes=[0, 5, 50, 1000]):
+    bio = six.BytesIO()
+    packer = Packer()
+    for size in sizes:
+        bio.write(packer.pack_map_header(size))
+        for i in range(size):
+            bio.write(packer.pack(i)) # key
+            bio.write(packer.pack(i * 2)) # value
+
+    bio.seek(0)
+    unpacker = Unpacker(bio)
+    for size in sizes:
+        assert unpacker.unpack() == dict((i, i * 2) for i in range(size))
 
 
 class odict(dict):
@@ -109,11 +149,15 @@ class odict(dict):
 def test_odict():
     seq = [(b'one', 1), (b'two', 2), (b'three', 3), (b'four', 4)]
     od = odict(seq)
-    assert_equal(unpackb(packb(od), use_list=1), dict(seq))
+    assert unpackb(packb(od), use_list=1) == dict(seq)
     def pair_hook(seq):
-        return seq
-    assert_equal(unpackb(packb(od), object_pairs_hook=pair_hook, use_list=1), seq)
+        return list(seq)
+    assert unpackb(packb(od), object_pairs_hook=pair_hook, use_list=1) == seq
 
 
-if __name__ == '__main__':
-    main()
+def test_pairlist():
+    pairlist = [(b'a', 1), (2, b'b'), (b'foo', b'bar')]
+    packer = Packer()
+    packed = packer.pack_map_pairs(pairlist)
+    unpacked = unpackb(packed, object_pairs_hook=list)
+    assert pairlist == unpacked
